from nuaal.Parsers.PatternsLib import PatternsLib
from nuaal.Parsers.Parser import ParserModule
from nuaal.Parsers.CiscoIOSParser import CiscoIOSParser
from nuaal.Parsers.JuniperJUNOSParser import JuniperJUNOSParser